﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' Draw some text along a line segment.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAliasGridFit
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias
        e.Graphics.InterpolationMode = InterpolationMode.High

        ' Draw some text along some paths.
        Dim path As New GraphicsPath()
        path.AddArc(New RectangleF(40, 40, 320, 220), 180, 180)
        e.Graphics.DrawPath(Pens.Green, path)
        DrawTextOnPath(e.Graphics, Brushes.Blue, Me.Font, _
                "This is some text drawn along a path", path, True)
        DrawTextOnPath(e.Graphics, Brushes.Blue, Me.Font, _
                "This is some text drawn along a path", path, False)

        path = New GraphicsPath()
        path.AddArc(New RectangleF(40, 50, 320, 220), 0, 180)
        e.Graphics.DrawPath(Pens.Red, path)
        DrawTextOnPath(e.Graphics, Brushes.Blue, Me.Font, _
                "This is some text drawn along a path", path, True)
        DrawTextOnPath(e.Graphics, Brushes.Blue, Me.Font, _
                "This is some text drawn along a path", path, False)
    End Sub

    ' Draw some text along a GraphicsPath.
    Private Sub DrawTextOnPath(ByVal gr As Graphics, ByVal brush As Brush, ByVal font As Font, ByVal txt As String, ByVal path As GraphicsPath, ByVal text_above_path As Boolean)
        ' Make a copy so we don't mess up the original.
        path = DirectCast(path.Clone(), GraphicsPath)

        ' Flatten the path into segments.
        path.Flatten()

        ' Draw characters.
        Dim start_ch As Integer = 0
        Dim start_point As PointF = path.PathPoints(0)
        For i As Integer = 1 To path.PointCount - 1
            Dim end_point As PointF = path.PathPoints(i)
            DrawTextOnSegment(gr, brush, font, txt, start_ch, _
                start_point, end_point, text_above_path)
            If (start_ch >= txt.Length) Then Exit For
        Next i
    End Sub

    ' Draw some text along a line segment.
    ' Leave char_num pointing to the next character to be drawn.
    ' Leave start_point holding the coordinates of the last point used.
    Private Sub DrawTextOnSegment(ByVal gr As Graphics, ByVal brush As Brush, ByVal font As Font, ByVal txt As String, ByRef first_ch As Integer, ByRef start_point As PointF, ByVal end_point As PointF, ByVal text_above_segment As Boolean)
        Dim dx As Single = end_point.X - start_point.X
        Dim dy As Single = end_point.Y - start_point.Y
        Dim dist As Single = CSng(Math.Sqrt(dx * dx + dy * dy))
        dx /= dist
        dy /= dist

        ' See how many characters will fit.
        Dim last_ch As Integer = first_ch
        While (last_ch < txt.Length)
            Dim test_string As String = txt.Substring(first_ch, last_ch - first_ch + 1)
            If (gr.MeasureString(test_string, font).Width > dist) Then
                ' This is one too many characters.
                last_ch -= 1
                Exit While
            End If
            last_ch += 1
        End While
        If (last_ch < first_ch) Then Return
        If (last_ch >= txt.Length) Then last_ch = txt.Length - 1
        Dim chars_that_fit As String = txt.Substring(first_ch, last_ch - first_ch + 1)

        ' Rotate and translate to position the characters.
        Dim state As GraphicsState = gr.Save()
        If (text_above_segment) Then
            gr.TranslateTransform(0, _
                -gr.MeasureString(chars_that_fit, font).Height, _
                MatrixOrder.Append)
        End If
        Dim angle As Single = CSng(180 * Math.Atan2(dy, dx) / Math.PI)
        gr.RotateTransform(angle, MatrixOrder.Append)
        gr.TranslateTransform(start_point.X, start_point.Y, MatrixOrder.Append)

        ' Draw the characters that fit.
        gr.DrawString(chars_that_fit, font, brush, 0, 0)

        ' Restore the saved state.
        gr.Restore(state)

        ' Update first_ch and start_point.
        first_ch = last_ch + 1
        Dim text_width As Single = gr.MeasureString(chars_that_fit, font).Width
        start_point = New PointF( _
            start_point.X + dx * text_width, _
            start_point.Y + dy * text_width)
    End Sub
End Class
